import sys, os, random, pygame
sys.path.append(os.path.join("modules"))
import Player, PowerBar, Timer, HighScores, ParticleEmitter
from Resources import *
pygame.init()

if not pygame.mixer:
    print "Oops! Sound appears to be unavailable..."

size = width, height = 500, 500
screen = pygame.display.set_mode(size)
pygame.display.set_caption("Final Hope")


def main(difficulty=0):
    if difficulty == 0:
        drain = 0
        initalpower = 20
        emitter = 0.40
    elif difficulty == 1:
        drain = 0.02
        initalpower = 11
        emitter = 0.20
    elif difficulty == 2:
        drain = 0.04
        initalpower = 0
        emitter = 0.20
    
    background, background_rect = loadImage("bg.png")
    background = background.convert()
    
    landscape, landscape_rect = loadImage("landscapemask.png")
    landscape = landscape.convert()
    
    biosphere, biosphere_rect = loadImage("biosphere.png")
    biosphere = biosphere.convert_alpha()
    biosphere_rect = biosphere_rect.move(130, 10)
    
    biosphere_entrance = pygame.Rect((biosphere_rect.centerx - 25, 
                                      biosphere_rect.top + 118, 50, 50))
    
    bridge0, bridge0_rect = loadImage("bridge.png")
    bridge0 = bridge0.convert_alpha()
    bridge0_rect = bridge0_rect.move(310, 200)
    
    bridge1, bridge1_rect = loadImage("bridge.png")
    bridge1 = bridge1.convert_alpha()
    bridge1_rect = bridge1_rect.move(50, 330)
    
    powerleak, powerleak_rect = loadImage("powerleak.png")
    powerleak = powerleak.convert_alpha()
    powerleak_rect = powerleak_rect.move(270, 430)
    
    powerpack, powerpack_rect = loadImage("powerpack.png")
    powerpack = powerpack.convert_alpha()
    powerpack_rect = powerpack_rect.move(450, 450)
    drawPowerPack1 = True
    
    emitter1 = ParticleEmitter.ParticleEmitter(280, 440, angle=100, gravity=.04, delay=1, spread=20, color=(255, 255, 0, 255), numToEmit=5, speed=2, lifetime=80, maxParticles=40)
    emitter1.start()
    
    # Draw the scene to the screen.
    screen.blit(background, background_rect)
    screen.blit(biosphere, biosphere_rect)
    screen.blit(bridge0, bridge0_rect)
    screen.blit(bridge1, bridge1_rect)
    screen.blit(powerleak, powerleak_rect)
    screen.blit(powerpack, powerpack_rect)
    pygame.display.flip()
    
    # Loading sprites and media.
    timer = Timer.Timer()
    player = Player.Player()
    power = PowerBar.PowerBar(initalpower, drain)
    music = loadMusic("FinalHopeTheme1.ogg")
    theSprites = pygame.sprite.RenderPlain((player, power))
    clock = pygame.time.Clock()
    
    while True:
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                fadeout()
                return power, None
            if event.type == pygame.KEYDOWN:
                if event.key == pygame.K_ESCAPE:
                    fadeout()
                    return power, None

        keystate = pygame.key.get_pressed()
        if keystate[pygame.K_SPACE]:
            if power.getPower() > 0.1:
                jet = True
                power.decreasePower(0.1)
            else:
                jet = False
        else:
            jet = False
        if keystate[pygame.K_UP]:
            player.moveUp(landscape, jet)
        if keystate[pygame.K_DOWN]:
            player.moveDown(landscape, jet)
        if keystate[pygame.K_LEFT]:
            player.moveLeft(landscape, jet)
        if keystate[pygame.K_RIGHT]:
            player.moveRight(landscape, jet)
        
        if(timer.getTime() == 0):
            fadeout()
            return power, timer
        
        # Returned to the biosphere. End of game.
        if biosphere_entrance.contains(player.getRect()):
            fadeout()
            return power, timer
            
        # Put in the adjustments to PowerBar here...
        if powerleak_rect.colliderect(player.getRect()):
            power.increasePower(emitter)
            
        # Power Pack powerup here...
        if drawPowerPack1:
            if powerpack_rect.colliderect(player.getRect()):
                power.increaseCapacity(50)
                drawPowerPack1 = False
        
        # Draw the scene to the screen. Again.
        screen.blit(background, background_rect)
        screen.blit(biosphere, biosphere_rect)
        screen.blit(bridge0, bridge0_rect)
        screen.blit(bridge1, bridge1_rect)
        screen.blit(powerleak, powerleak_rect)
        
        if drawPowerPack1:
            screen.blit(powerpack, powerpack_rect)
        
        emitter1.update()
        timer.update()
        
        theSprites.update()
        theSprites.draw(screen)
    
        pygame.display.flip()
        clock.tick(60)


def fadeout():
    for i in range(50):
        fader = pygame.Surface(screen.get_size())
        fader = fader.convert_alpha()
        fader.fill((0, 0, 0, i))
        screen.blit(fader, (0, 0))
        pygame.display.flip()
    return True


def showStory():
    story1, story1_rect = loadImage("logoscreenstory1.png")
    story1 = story1.convert()
    screen.blit(story1, (0, 0))
    pygame.display.flip()
    waitForInput = True
        
    while waitForInput == True:
        for event in pygame.event.get():
            if event.type == pygame.KEYDOWN or event.type == pygame.MOUSEBUTTONDOWN:
                fadeout()
                waitForInput = False
    
    story2, story2_rect = loadImage("logoscreenstory2.png")
    story2 = story2.convert()
    screen.blit(story2, (0, 0))
    pygame.display.flip()
    waitForInput = True
        
    while waitForInput == True:
        for event in pygame.event.get():
            if event.type == pygame.KEYDOWN or event.type == pygame.MOUSEBUTTONDOWN:
                fadeout()
                waitForInput = False
                

def mainMenu():
    menubg, menubg_rect = loadImage("logoscreenmenu.png")
    menubg = menubg.convert()
    screen.blit(menubg, (0, 0))
    pygame.display.flip()
    
    color = (0, 0, 190, 255)
    font = pygame.font.Font(os.path.join("data", "Brisk.ttf"), 36)
    
    difficulty = font.render("Please Click To ", 1, color)
    difficulty_rect = difficulty.get_rect()
    difficulty_rect = difficulty_rect.move(130, 200)

    difficulty2 = font.render("Choose A Difficulty:", 1, color)
    difficulty2_rect = difficulty2.get_rect()
    difficulty2_rect = difficulty2_rect.move(100, 230)
    
    easy = font.render("Easy", 1, color)
    easy_rect = easy.get_rect()
    easy_rect = easy_rect.move(200, 290)
    
    normal = font.render("Normal", 1, color)
    normal_rect = normal.get_rect()
    normal_rect = normal_rect.move(186, 330)
    
    hard = font.render("Hard", 1, color)
    hard_rect = hard.get_rect()
    hard_rect = hard_rect.move(200, 370)
    
    screen.blit(difficulty, difficulty_rect)
    screen.blit(difficulty2, difficulty2_rect)
    screen.blit(easy, easy_rect)
    screen.blit(normal, normal_rect)
    screen.blit(hard, hard_rect)
    pygame.display.flip()
    
    while True:
        for event in pygame.event.get():
            if event.type == pygame.KEYDOWN and event.key == pygame.K_ESCAPE:
                fadeout()
                sys.exit()
            if event.type == pygame.MOUSEBUTTONDOWN:
                mousepos = pygame.mouse.get_pos()
                if easy_rect.collidepoint(mousepos):
                    fadeout()
                    return 0
                if normal_rect.collidepoint(mousepos):
                    fadeout()
                    return 1
                if hard_rect.collidepoint(mousepos):
                    fadeout()
                    return 2


def askTut():
    menubg, menubg_rect = loadImage("logoscreenmenu.png")
    menubg = menubg.convert()
    screen.blit(menubg, (0, 0))
    pygame.display.flip()
    
    color = (0, 0, 190, 255)
    font = pygame.font.Font(os.path.join("data", "Brisk.ttf"), 36)
    
    line1 = font.render("Would you like to see", 1, color)
    line1_rect = line1.get_rect()
    line1_rect = line1_rect.move(90, 200)

    line2 = font.render("the tutorial?", 1, color)
    line2_rect = line2.get_rect()
    line2_rect = line2_rect.move(150, 230)
    
    yes = font.render("Yes", 1, color)
    yes_rect = yes.get_rect()
    yes_rect = yes_rect.move(180, 290)
    
    no = font.render("No", 1, color)
    no_rect = no.get_rect()
    no_rect = no_rect.move(260, 290)
    
    screen.blit(line1, line1_rect)
    screen.blit(line2, line2_rect)
    screen.blit(yes, yes_rect)
    screen.blit(no, no_rect)
    pygame.display.flip()
    
    while True:
        for event in pygame.event.get():
            if event.type == pygame.MOUSEBUTTONDOWN:
                mousepos = pygame.mouse.get_pos()
                if yes_rect.collidepoint(mousepos):
                    return True
                if no_rect.collidepoint(mousepos):
                    return False


def showTut():
    tut1, tut1_rect = loadImage("logoscreentut1.png")
    tut1 = tut1.convert()
    screen.blit(tut1, (0, 0))
    pygame.display.flip()
    waitForInput = True
        
    while waitForInput == True:
        for event in pygame.event.get():
            if event.type == pygame.KEYDOWN or event.type == pygame.MOUSEBUTTONDOWN:
                fadeout()
                waitForInput = False
    
    tut2, tut2_rect = loadImage("logoscreentut2.png")
    tut2 = tut2.convert()
    screen.blit(tut2, (0, 0))
    pygame.display.flip()
    waitForInput = True
        
    while waitForInput == True:
        for event in pygame.event.get():
            if event.type == pygame.KEYDOWN or event.type == pygame.MOUSEBUTTONDOWN:
                fadeout()
                waitForInput = False
    
    tut3, tut3_rect = loadImage("logoscreentut3.png")
    tut3 = tut3.convert()
    screen.blit(tut3, (0, 0))
    pygame.display.flip()
    waitForInput = True
        
    while waitForInput == True:
        for event in pygame.event.get():
            if event.type == pygame.KEYDOWN or event.type == pygame.MOUSEBUTTONDOWN:
                fadeout()
                waitForInput = False
                                    

def showScore(diff, power, timer):
    font = pygame.font.Font(os.path.join("data", "Brisk.ttf"), 20)
    color = (0, 0, 190, 255)
    text1 = []
    
    if timer == None:
        text1.append("Dirty quitter!")
        text1.append("People are dying back there!")
        totalScore = 0
    elif timer.getTime() == 0:
        text1.append("You gathered " + str(power.getPower()) + "% power.")
        text1.append("But you didn't return to the biosphere!")
        text1.append("")
        text1.append("Your Score:")
        powerScore = int(power.getPower() * 500 * (diff + 1) * 1.5)
        timerScore = int(timer.getTimeScore() * 100 * (diff + 1) * 1.5)
        totalScore = powerScore - timerScore
        stringy = " Power         [x500 ]:     " + str(powerScore)
        text1.append(stringy)
        stringy = "-Time Taken   [x100 ]:     " + str(timerScore)
        text1.append(stringy)
        stringy = "-------------------------------------------"
        text1.append(stringy)
        stringy = " Total Score:                        " + str(totalScore)
        text1.append(stringy)
    else:
        text1.append("Welcome home.")
        text1.append("You gathered " + str(power.getPower()) + "% power.")
        text1.append("You took " + str(timer.getTimeTaken()) + " seconds.")
        text1.append("")
        text1.append("Your Score:")
        powerScore = int(power.getPower() * 1000 * (diff + 1) * 1.5)
        timerScore = int(timer.getTimeScore() * 100 * (diff + 1) * 1.5)
        totalScore = powerScore - timerScore
        stringy = " Power         [x1,000]:     " + str(powerScore)
        text1.append(stringy)
        stringy = "-Time Taken   [x100 ]:     " + str(timerScore)
        text1.append(stringy)
        stringy = "-------------------------------------------"
        text1.append(stringy)
        stringy = " Total Score:                        " + str(totalScore)
        text1.append(stringy)
    
    background, background_rect = loadImage("logoscreenmenu.png")
    background = background.convert()
    screen.blit(background, (0, 0))
    
    initalX, initalY = 100, 160
    for line in text1:
        text = font.render(line, 1, color)
        textpos = text.get_rect()
        textpos = textpos.move(initalX, initalY + (text1.index(line) * 30))
        screen.blit(text, textpos)
    
    pygame.display.flip()
        
    while True:
        for event in pygame.event.get():
            if event.type == pygame.KEYDOWN and event.key == pygame.K_ESCAPE:
                fadeout()
                return totalScore
            if event.type == pygame.MOUSEBUTTONDOWN:
                fadeout()
                return totalScore
                

def getUserName():
    font = pygame.font.Font(os.path.join("data", "Brisk.ttf"), 26)
    color = (0, 0, 190, 255)
    user = ""
    allKeys = [pygame.K_a, pygame.K_b, pygame.K_c, pygame.K_d, 
               pygame.K_e, pygame.K_f, pygame.K_g, pygame.K_h, pygame.K_i, 
               pygame.K_j, pygame.K_k, pygame.K_l, pygame.K_m, pygame.K_n, 
               pygame.K_o, pygame.K_p, pygame.K_q, pygame.K_r, pygame.K_s, 
               pygame.K_t, pygame.K_u, pygame.K_v, pygame.K_w, pygame.K_x, 
               pygame.K_y, pygame.K_z]
    allLetters = ["A", "B", "C", "D", "E", "F", "G", "H", "I", "J", "K",
                  "L", "M", "N", "O", "P", "Q", "R", "S", "T", "U", "V",
                  "W", "X", "Y", "Z"]
    
    background, background_rect = loadImage("logoscreenmenu.png")
    background = background.convert()
    screen.blit(background, (0, 0))
    
    prompt = font.render("Please enter your name:", 1, color)
    screen.blit(prompt, (110, 200))
    
    pygame.display.flip()

    while len(user) < 9:
        for event in pygame.event.get():
            if event.type == pygame.KEYDOWN and \
               event.key in allKeys:
                user += allLetters[allKeys.index(event.key)]
            if event.type == pygame.KEYDOWN and event.key == pygame.K_RETURN:
                return user
            if event.type == pygame.KEYDOWN and event.key == pygame.K_BACKSPACE:
                user = user[:-1]
        
        screen.blit(background, (0, 0))
        screen.blit(prompt, (110, 200))
        
        text = font.render(user, 1, color)
        textpos = text.get_rect()
        textpos = textpos.move(180, 230)
        screen.blit(text, textpos)
        
        pygame.display.flip()
    
    return user

                

def showHighScores(totalScore):
    hs = HighScores.HighScores()

    lowscore = hs.getLowScore()
    if totalScore > lowscore:
        name = getUserName()
        hs.addScore(name, totalScore)
        hs.writeFile()
    
    background, background_rect = loadImage("logoscreen.png")
    background = background.convert()
    screen.blit(background, (0, 0))
    
    hs.drawToScreen(80, 120)
    pygame.display.flip()
        
    while True:
        for event in pygame.event.get():
            if event.type == pygame.KEYDOWN and event.key == pygame.K_ESCAPE:
                fadeout()
                return
            if event.type == pygame.MOUSEBUTTONDOWN:
                fadeout()
                return
                

def askKeepRunning():
    menubg, menubg_rect = loadImage("logoscreenmenu.png")
    menubg = menubg.convert()
    screen.blit(menubg, (0, 0))
    pygame.display.flip()
    
    color = (0, 0, 190, 255)
    font = pygame.font.Font(os.path.join("data", "Brisk.ttf"), 36)
    
    line1 = font.render("Would you like to", 1, color)
    line1_rect = line1.get_rect()
    line1_rect = line1_rect.move(110, 200)

    line2 = font.render("play again?", 1, color)
    line2_rect = line2.get_rect()
    line2_rect = line2_rect.move(160, 230)
    
    yes = font.render("Yes", 1, color)
    yes_rect = yes.get_rect()
    yes_rect = yes_rect.move(180, 290)
    
    no = font.render("No", 1, color)
    no_rect = no.get_rect()
    no_rect = no_rect.move(260, 290)
    
    screen.blit(line1, line1_rect)
    screen.blit(line2, line2_rect)
    screen.blit(yes, yes_rect)
    screen.blit(no, no_rect)
    pygame.display.flip()
    
    while True:
        for event in pygame.event.get():
            if event.type == pygame.MOUSEBUTTONDOWN:
                mousepos = pygame.mouse.get_pos()
                if yes_rect.collidepoint(mousepos):
                    return True
                if no_rect.collidepoint(mousepos):
                    return False
                

if __name__ == "__main__":
    keepRunning = True
    showStory()
    if askTut():
        showTut()
        
    while keepRunning == True:
        diff = mainMenu()
        power, timer = main(diff)
        totalScore = showScore(diff, power, timer)
        showHighScores(totalScore)
        keepRunning = askKeepRunning()
    sys.exit()