import pygame
import Particle
from Resources import *

class ParticleEmitter:
    """An emitter of particles."""
    def __init__(self, xLoc, yLoc, angle=90, speed=1, gravity=0,
                 lifetime=50, spread=0, delay=10, maxParticles=10,
                 image=None, color=(255, 255, 255, 255), numToEmit=1, fade=0):
        self.particles = []
        self.xLoc = xLoc
        self.yLoc = yLoc
        self.angle = angle
        self.speed = speed
        self.gravity = gravity
        self.lifetime = lifetime
        self.spread = spread
        self.delay = delay
        self.maxParticles = maxParticles
        self.image = image
        self.color = color
        self.numToEmit = numToEmit
        self.fade = fade
        self.run = False

    def start(self):
        self.run = True
        self.delayCount = 1

    def stop(self):
        self.run = False
        
    def move(self, x, y):
        self.xLoc = x
        self.yLoc = y
        
    def changeAngle(self, angle):
        self.angle = angle
        
    def setMaxParts(self, max):
        self.maxParticles = max

    def update(self):
        if self.run == True:
            if self.delayCount == self.delay:
                if len(self.particles) < self.maxParticles:    
                    for i in range(self.numToEmit):
                        newpart = Particle.Particle(self.xLoc, self.yLoc,
                                           self.angle, self.speed, self.gravity, 
                                           self.lifetime, self.spread,
                                           self.image, self.color, self.fade)
                        self.particles.append(newpart)
            
            self.delayCount += 1
            
            if self.delayCount > self.delay:
                self.delayCount = 0

            for part in self.particles:
                    alive = part.update()
                    if not alive:
                        theIndex = self.particles.index(part)
                        self.particles = self.particles[:theIndex] + self.particles[theIndex + 1:]


if __name__ == "__main__":
    pygame.init()
    size = width, height = 300, 300
    screen = pygame.display.set_mode(size)
    pygame.display.set_caption("Particle Test")

    background = pygame.Surface(screen.get_size())
    background.fill((0, 0, 0))
    background = background.convert()

    emitter1 = ParticleEmitter(280, 280, angle=110, gravity=.04, delay=1, spread=20, color=(0, 0, 255, 255), numToEmit=5, speed=2, lifetime=80, maxParticles=40)
    emitter1.start()
    emitter2 = ParticleEmitter(50, 50, angle=0, gravity=0, delay=0, spread=100, color=(255, 0, 0, 255), numToEmit=30, speed=0, lifetime=100, maxParticles=50, fade=8)
    emitter2.start()
    emitter3 = ParticleEmitter(10, 250, angle=45, gravity=.02, delay=5, spread=0, color=(255, 255, 255, 255), numToEmit=1, speed=2, lifetime=120, maxParticles=200, fade=4)
    emitter3.start()
    emitter4 = ParticleEmitter(240, 100, angle=90, gravity=-.05, delay=1, spread=10, color=(255, 200, 0, 100), numToEmit=10, speed=.3, lifetime=150, maxParticles=1000, fade=6)
    emitter4.start()
    
    clock = pygame.time.Clock()
    run = True
    
    while run == True:
        for event in pygame.event.get():
            if event.type == pygame.KEYDOWN and event.key == pygame.K_ESCAPE:
                emitter1.stop()
                emitter2.stop()
                emitter3.stop()
                emitter4.stop()
                run = False
        
        screen.blit(background, (0, 0))
        emitter1.update()
        emitter2.update()
        emitter3.update()
        emitter4.update()
        pygame.display.flip()
        clock.tick(120)
